# $File: //member/autrijus/MasonX-Profiler/t/1-basic.t $ $Author: autrijus $
# $Revision: #1 $ $Change: 8431 $ $DateTime: 2003/10/16 11:21:57 $

use strict;
use Test;
BEGIN { plan tests => 1 }

use Parse::Binary; ok(1);

1;
